"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PathSelector = exports.createReactiveState = void 0;

var _mobx = require("mobx");

var _get = _interopRequireDefault(require("lodash/get"));

var _set = _interopRequireDefault(require("lodash/set"));

var _merge = _interopRequireDefault(require("lodash/merge"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const createReactiveState = (initialValues = {}) => {
  const store = _mobx.observable.object(initialValues);

  const subscribe = (selector, onChange) => {
    (0, _mobx.autorun)(() => {
      onChange(selector(store));
    });
  };

  const set = (path, value) => {
    (0, _set.default)(store, path, value);
    return state;
  };

  const assign = values => {
    Object.assign(store, values);
    return state;
  };

  const merge = values => {
    merge(store, values);
    return state;
  };

  const state = {
    store,
    subscribe,
    set,
    assign,
    merge
  };
  return state;
};

exports.createReactiveState = createReactiveState;

const PathSelector = paths => store => {
  return paths.reduce((values, path) => {
    return set(values, path, (0, _get.default)(store, path));
  }, {});
};

exports.PathSelector = PathSelector;